/*****************************************************************************
 *
 * LOGGING.C - Log file functions for use with NetSaint
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   12-14-1999
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netsaint.h"

extern char	log_file[MAX_FILENAME_LENGTH];
extern char     status_file[MAX_FILENAME_LENGTH];
extern char     temp_file[MAX_FILENAME_LENGTH];
extern char	log_archive_path[MAX_FILENAME_LENGTH];

extern char     macro_current_service_attempt[MAX_ATTEMPT_LENGTH];
extern char	macro_service_state[MAX_STATE_LENGTH];

extern int	log_level;
extern int	use_syslog;
extern int	syslog_level;
extern int      log_service_retries;

extern int      verify_config;

extern time_t   program_start;
extern int      program_mode;
extern time_t   last_mode_change;
extern int      daemon_mode;

extern int      total_orphaned_children;
extern time_t   last_zombie_cleanup;

extern time_t   last_command_check;

extern int      zombie_cleanup_interval;
extern int      command_check_interval;

extern int      log_rotation_method;
extern time_t   last_log_rotation;


extern host		*host_list;
extern service		*service_list;




/******************************************************************/
/************************ LOGGING FUNCTIONS ***********************/
/******************************************************************/

/* write something to the netsaint log file */
int write_to_log(char *buffer){
	FILE *fp;
	time_t t;

#ifdef DEBUG0
	printf("write_to_log() start\n");
#endif

	/* don't log anything if we are just verifying the configuration */
	if(verify_config==TRUE)
		return OK;

	/* make sure log file has been specified */
	if(!strcmp(log_file,"")){
		printf("Warning: Log file has not been specified - cannot write to log!\n");
		return ERROR;
	        }

	fp=fopen(log_file,"a+");
	if(fp==NULL){
		printf("Warning: Cannot open log file '%s' for writing\n",log_file);
		return ERROR;
		}

	/* get the current time */
	time(&t);

	/* strip any newlines from the end of the buffer */
	strip(buffer);

	/* write the buffer to the log file */
	fprintf(fp,"[%lu] %s\n",t,buffer);

	fclose(fp);

#ifdef DEBUG0
	printf("write_to_log() end\n");
#endif

	return OK;
	}


/* write something to the syslog facility */
int write_to_syslog(char *buffer){

#ifdef DEBUG0
	printf("write_to_syslog() start\n");
#endif

	/* bail out if we shouldn't write to syslog */
	if(!use_syslog)
		return OK;

	/* open the syslog */
	syslog(LOG_USER|LOG_INFO,"%s",buffer);

#ifdef DEBUG0
	printf("write_to_syslog() end\n");
#endif

	return OK;
	}


/* write a service problem/recovery to the netsaint log file */
int log_service_event(service *svc,int state_type){
	char temp_buffer[MAX_INPUT_BUFFER];
	int write_data_to_log=TRUE;
	int write_data_to_syslog=TRUE;

#ifdef DEBUG0
	printf("log_service_event() start\n");
#endif

	/* don't log soft errors if the user doesn't want to */
	if(state_type==SOFT_STATE && !log_service_retries)
		return OK;

	/* don't log warnings or unknown errors if we only report critical stuff */
	if((svc->current_state==STATE_WARNING || svc->current_state==STATE_UNKNOWN) && log_level==2)
		write_data_to_log=FALSE;
	if((svc->current_state==STATE_WARNING || svc->current_state==STATE_UNKNOWN) && syslog_level==2)
		write_data_to_syslog=FALSE;

	/* grab service macros */
	grab_service_macros(svc);

	snprintf(temp_buffer,sizeof(temp_buffer),"SERVICE ALERT: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,macro_service_state,(state_type==SOFT_STATE)?"SOFT":"HARD",macro_current_service_attempt,svc->plugin_output);
	temp_buffer[sizeof(temp_buffer)-1]='\x0';

	if(write_data_to_log==TRUE)
		write_to_log(temp_buffer);
	if(write_data_to_syslog==TRUE && use_syslog)
		write_to_syslog(temp_buffer);

#ifdef DEBUG0
	printf("log_service_event() end\n");
#endif

	return OK;
	}




/* update the status log with the current status of a particular services */
int update_status_log_with_service(service *svc){
	FILE *fp,*fp2;
	char input_buffer[MAX_INPUT_BUFFER];
	char state_string[16];
	char last_hard_state_string[16];
	char service_string[MAX_INPUT_BUFFER];
	host *temp_host;
	int rename_result=0;
	time_t next_check;

#ifdef DEBUG0
	printf("update_status_log_with_service() start\n");
#endif

	/* get the host associated with this service */
	temp_host=NULL;
	temp_host=find_host(svc->host_name,temp_host);

	/* update the status log with the host info */
	update_status_log_with_host(temp_host);

	/* get the service state string */
	if(temp_host->status==HOST_DOWN)
		strcpy(state_string,"HOST DOWN");
	else if(temp_host->status==HOST_UNREACHABLE)
		strcpy(state_string,"UNREACHABLE");
	else if(svc->current_state==STATE_OK){
		if(svc->last_state==STATE_OK)
			strcpy(state_string,"OK");
		else
			strcpy(state_string,"RECOVERY");
	        }
	else if(svc->current_state==STATE_CRITICAL)
		strcpy(state_string,"CRITICAL");
	else if(svc->current_state==STATE_WARNING)
		strcpy(state_string,"WARNING");
	else
		strcpy(state_string,"UNKNOWN");

	if(svc->last_hard_state==STATE_OK)
		strcpy(last_hard_state_string,"OK");
	else if(svc->last_hard_state==STATE_CRITICAL)
		strcpy(last_hard_state_string,"CRITICAL");
	else if(svc->last_hard_state==STATE_WARNING)
		strcpy(last_hard_state_string,"WARNING");
	else
		strcpy(last_hard_state_string,"UNKNOWN");


	/* create a service string to identify previous entries in the log for this service/host pair */
	snprintf(service_string,sizeof(service_string),"] SERVICE;%s;%s;",svc->host_name,svc->description);
	service_string[sizeof(service_string)-1]='\x0';

	/* create a temp file for writing the new status log */
	fp2=fopen(temp_file,"w");
	if(fp2==NULL){
		printf("Error: Could not create temp file '%s'!\n",temp_file);
		return ERROR;
	        }

	/* open the existing status log */
	fp=fopen(status_file,"r");
	if(fp!=NULL){

		/* read all lines from the original status log */
		while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

			/* if this is the old service string, write the new service info instead */
			if(strstr(input_buffer,service_string)){

				if((unsigned long)svc->last_check==0L){
					next_check=(time_t)svc->next_check;
					fprintf(fp2,"[N/A] SERVICE;%s;%s;PENDING;0/%d;HARD;N/A;%d;N/A;OK;0;0;0;0;N/A;%d;Service check scheduled for %s\n",svc->host_name,svc->description,svc->max_attempts,(svc->disabled==TRUE)?0:1,(svc->notifications_disabled==TRUE)?0:1,ctime(&next_check)); 
				        }
				else{
					fprintf(fp2,"[%lu] SERVICE;",(unsigned long)svc->last_check);
					fprintf(fp2,"%s;%s;%s;",svc->host_name,svc->description,state_string);
					fprintf(fp2,"%d/%d;",(svc->state_type==SOFT_STATE)?svc->current_attempt-1:svc->current_attempt,svc->max_attempts);
					fprintf(fp2,"%s;",(svc->state_type==SOFT_STATE)?"SOFT":"HARD");
					fprintf(fp2,"%lu;%d;",(unsigned long)svc->next_check,(svc->disabled==TRUE)?0:1);
					fprintf(fp2,"%lu;%s;",(unsigned long)svc->last_state_change,last_hard_state_string);
					fprintf(fp2,"%lu;%lu;%lu;%lu;",svc->time_ok,svc->time_unknown,svc->time_warning,svc->time_critical);
					fprintf(fp2,"%lu;%d;",(unsigned long)svc->last_notification,(svc->notifications_disabled==TRUE)?0:1);
					fprintf(fp2,"%s\n",svc->plugin_output);
				        }
			        }

			/* if this is not a duplicate entry, write it to the new log file */
			else
				fputs(input_buffer,fp2);
		        }

		fclose(fp);
	        }
	else{
		printf("Error: Could not open status log '%s' for reading!\n",status_file);
		return ERROR;
	        }

	fclose(fp2);

	/* move the temp file to the status log (overwrite the old status log) */
	rename_result=rename(temp_file,status_file);

	if(rename_result){
		printf("Error: Could not rename() '%s' to '%s'\n",temp_file,status_file);
		return ERROR;
	        }

#ifdef DEBUG0
	printf("update_status_log_with_service() end\n");
#endif

	return OK;
        }




/* update the status log with the current status of a particular host */
int update_status_log_with_host(host *hst){
	FILE *fp,*fp2;
	char input_buffer[MAX_INPUT_BUFFER];
	char state_string[16];
	char host_string[MAX_INPUT_BUFFER];
	int rename_result=0;
	time_t current_time;

#ifdef DEBUG0
	printf("update_status_log_with_host() start\n");
#endif

	/* get the host state string */
	if(hst->status==HOST_DOWN)
		strcpy(state_string,"DOWN");
	else if(hst->status==HOST_UNREACHABLE)
		strcpy(state_string,"UNREACHABLE");
	else
		strcpy(state_string,"UP");

	time(&current_time);

	/* create a host string to identify previous entries in the log file for this host */
	snprintf(host_string,sizeof(host_string),"] HOST;%s;",hst->name);
	host_string[sizeof(host_string)-1]='\x0';

	/* create a temp file for writing the new status log */
	fp2=fopen(temp_file,"w");
	if(fp2==NULL){
		printf("Error: Could not create temp file '%s'!\n",temp_file);
		return ERROR;
	        }

	/* open the existing status log */
	fp=fopen(status_file,"r");
	if(fp!=NULL){

		/* read all lines from the original status log */
		while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

			/* if this is the old host string, write the new host info instead */
			if(strstr(input_buffer,host_string)){

				if((unsigned long)hst->last_check==0L)
					fprintf(fp2,"[N/A] HOST;%s;PENDING;N/A;0;0;0;N/A;%d;(No plugin output available yet)\n",hst->name,(hst->notifications_disabled==TRUE)?0:1);
				else{
					fprintf(fp2,"[%lu] HOST;",(unsigned long)current_time);
					fprintf(fp2,"%s;%s;",hst->name,state_string);
					fprintf(fp2,"%lu;",(unsigned long)hst->last_state_change);
					fprintf(fp2,"%lu;%lu;%lu;",hst->time_up,hst->time_down,hst->time_unreachable);
					fprintf(fp2,"%lu;%d;",(unsigned long)hst->last_host_notification,(hst->notifications_disabled==TRUE)?0:1);
					fprintf(fp2,"%s\n",hst->plugin_output);
				        }
			        }

			/* if this is not a duplicate entry, write it to the new log file */
			else
				fputs(input_buffer,fp2);
		        }

		fclose(fp);
	        }
	else{
		printf("Error: Could not open status log '%s' for reading!\n",status_file);
		return ERROR;
	        }

	fclose(fp2);

	/* move the temp file to the status log (overwrite the old status log) */
	rename_result=rename(temp_file,status_file);

	if(rename_result){
		printf("Error: Could not rename() '%s' to '%s'\n",temp_file,status_file);
		return ERROR;
	        }

#ifdef DEBUG0
	printf("update_status_log_with_host() end\n");
#endif

	return OK;
        }



/* update the status log with the current program information */
int update_status_log_with_program(void){
	FILE *fp,*fp2;
	char input_buffer[MAX_INPUT_BUFFER];
	char program_string[MAX_INPUT_BUFFER];
	char mode_string[8];
	int rename_result=0;
	time_t current_time;

#ifdef DEBUG0
	printf("update_status_log_with_program() start\n");
#endif

	time(&current_time);

	/* get the program_mode string */
	if(program_mode==STANDBY_MODE)
		strcpy(mode_string,"STANDBY");
	else
		strcpy(mode_string,"ACTIVE");

	/* create a string to identify previous entries in the log file for the program info */
	snprintf(program_string,sizeof(program_string),"] PROGRAM;");
	program_string[sizeof(program_string)-1]='\x0';

	/* create a temp file for writing the new status log */
	fp2=fopen(temp_file,"w");
	if(fp2==NULL){
		printf("Error: Could not create temp file '%s'!\n",temp_file);
		return ERROR;
	        }

	/* open the existing status log */
	fp=fopen(status_file,"r");
	if(fp!=NULL){

		/* read all lines from the original status log */
		while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

			/* if this is the old program string, write the new program info instead */
			if(strstr(input_buffer,program_string)){

				fprintf(fp2,"[%lu] PROGRAM;",(unsigned long)current_time);
				fprintf(fp2,"%lu;%d;",(unsigned long)program_start,(daemon_mode==TRUE)?1:0);
				fprintf(fp2,"%s;%lu;",mode_string,(unsigned long)last_mode_change);
				fprintf(fp2,"%lu;%d;",(unsigned long)last_zombie_cleanup,total_orphaned_children);
				fprintf(fp2,"%lu;%lu\n",(unsigned long)last_command_check,(unsigned long)last_log_rotation);
			        }

			/* if this is not a duplicate entry, write it to the new log file */
			else
				fputs(input_buffer,fp2);
		        }

		fclose(fp);
	        }
	else{
		printf("Error: Could not open status log '%s' for reading!\n",status_file);
		return ERROR;
	        }

	fclose(fp2);

	/* move the temp file to the status log (overwrite the old status log) */
	rename_result=rename(temp_file,status_file);

	if(rename_result){
		printf("Error: Could not rename() '%s' to '%s'\n",temp_file,status_file);
		return ERROR;
	        }

#ifdef DEBUG0
	printf("update_status_log_with_program() end\n");
#endif

	return OK;
        }


int create_initial_status_log(void){
	service *temp_service;
	host *temp_host;
	char temp_buffer[MAX_INPUT_BUFFER];
	time_t next_check;
	time_t current_time;
	FILE *fp;

#ifdef DEBUG0
	printf("create_initial_status_log() start\n");
#endif

	/* open the status log for writing */
	fp=fopen(status_file,"w");
	if(fp==NULL){

#ifdef DEBUG1
		printf("Error: Could not open status log for writing\n");
#endif

		return ERROR;
	        }

	/* add the program start identifier */
	time(&current_time);
	snprintf(temp_buffer,sizeof(temp_buffer),"[%lu] PROGRAM;%lu;%d;%s;%lu;N/A;0;N/A;N/A\n",current_time,program_start,(daemon_mode==TRUE)?1:0,(program_mode==ACTIVE_MODE)?"ACTIVE":"STANDBY",last_mode_change);
	temp_buffer[sizeof(temp_buffer)-1]='\x0';
	fputs(temp_buffer,fp);

	/* loop through all the hosts */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		snprintf(temp_buffer,sizeof(temp_buffer),"[N/A] HOST;%s;PENDING;N/A;0;0;0;N/A;1;(No plugin output available yet)\n",temp_host->name);
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		fputs(temp_buffer,fp);
	        }

	/* loop through all the services */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		next_check=(time_t)temp_service->next_check;
		snprintf(temp_buffer,sizeof(temp_buffer),"[N/A] SERVICE;%s;%s;PENDING;0/%d;HARD;N/A;1;N/A;OK;0;0;0;0;N/A;1;Service check scheduled for %s",temp_service->host_name,temp_service->description,temp_service->max_attempts,ctime(&next_check)); 
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		fputs(temp_buffer,fp);
                }

	/* close the status log */
	fclose(fp);

#ifdef DEBUG0
	printf("create_initial_status_log() end\n");
#endif

	return OK;
        }




/* rotates the main log file */
int rotate_log_file(void){
	char temp_buffer[MAX_INPUT_BUFFER];
	char method_string[16];
	char log_archive[MAX_FILENAME_LENGTH];
	time_t current_time;
	struct tm *t;
	int rename_result;

#ifdef DEBUG0
	printf("rotate_log_file() start\n");
#endif

	if(log_rotation_method==LOG_ROTATION_NONE){

#ifdef DEBUG1
		printf("\tWe're not supposed to be doing log rotations!\n");
#endif
		return OK;
	        }
	else if(log_rotation_method==LOG_ROTATION_HOURLY)
		strcpy(method_string,"HOURLY");
	else if(log_rotation_method==LOG_ROTATION_DAILY)
		strcpy(method_string,"DAILY");
	else if(log_rotation_method==LOG_ROTATION_WEEKLY)
		strcpy(method_string,"WEEKLY");
	else if(log_rotation_method==LOG_ROTATION_MONTHLY)
		strcpy(method_string,"MONTHLY");
	else
		return ERROR;

	/* update the last log rotation time and status log */
	last_log_rotation=time(NULL);
	update_status_log_with_program();

	/* get the current time */
	time(&current_time);
	t=localtime(&current_time);

	/* get the archived filename to use */
	snprintf(log_archive,sizeof(log_archive),"%s%snetsaint-%02d-%02d-%d-%02d.log",log_archive_path,(log_archive_path[strlen(log_archive_path)-1]=='/')?"":"/",t->tm_mon+1,t->tm_mday,t->tm_year+1900,t->tm_hour);
	log_archive[sizeof(log_archive)-1]='\x0';

	/* rotate the log file */
	rename_result=rename(log_file,log_archive);

	if(rename_result){

#ifdef DEBUG1
		printf("\tError: Could not rotate main log file to '%s'\n",log_archive);
#endif

		return ERROR;
	        }

	/* record the log rotation after it has been done... */
	snprintf(temp_buffer,sizeof(temp_buffer),"LOG ROTATION: %s\n",method_string);
	temp_buffer[sizeof(temp_buffer)-1]='\x0';
	write_to_log(temp_buffer);


#ifdef DEBUG3
	printf("\tRotated main log file to '%s'\n",log_archive);
#endif

#ifdef DEBUG0
	printf("rotate_log_file() end\n");
#endif

	return OK;
        }
