/*****************************************************************************
 *
 * NOTIFICATIONS.C - Service and host notification functions for NetSaint
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   02-18-2000
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netsaint.h"

extern notification    *notification_list;
extern hostgroup       *hostgroup_list;

extern int             program_mode;
extern int             interval_length;
extern int             log_notifications;

extern char            macro_summary[MAX_COMMANDLINE_LENGTH];
extern char            generic_summary[MAX_COMMANDLINE_LENGTH];
extern char            macro_host_state[MAX_STATE_LENGTH];



/******************************************************************/
/***************** SERVICE NOTIFICATION FUNCTIONS *****************/
/******************************************************************/

/* notify contacts about a service problem or recovery */
int service_notification(service *svc){
	host *temp_host;
	notification *temp_notification;
	time_t current_time;
	time_t timeperiod_start;

#ifdef DEBUG0
	printf("service_notification() start\n");
#endif

	/* are we in active mode? */
	if(program_mode!=ACTIVE_MODE){
#ifdef DEBUG4
		printf("\tWe are in STANDBY mode, so service notifications will not be sent out!\n");
#endif
		return OK;
	        }

	/* are notifications temporarily disabled for this service? */
	if(svc->notifications_disabled==TRUE){
#ifdef DEBUG4
		printf("\tNotifications are temporarily disabled for this service, so we won't send one out!\n");
#endif
		return OK;
	        }

	/* find the host this service is associated with */
	temp_host=find_host(svc->host_name,NULL);

	/* if we couldn't find the host, return an error */
	if(temp_host==NULL)
		return ERROR;

	time(&current_time);

	/* if the host is down or unreachable, don't notify contacts about multiple service failures */
	if(temp_host->status!=HOST_UP){
#ifdef DEBUG4
		printf("\tThe host is either down or unreachable, so we won't notify contacts about this service!\n");
#endif
		return OK;
	        }
	
	/* see if we should notify about problems with this service */
	if((svc->current_state==STATE_WARNING || svc->current_state==STATE_UNKNOWN) && !svc->notify_on_warning){
#ifdef DEBUG4
		printf("\tWe shouldn't notify about WARNING or UNKNOWN states for this service!\n");
#endif
		return OK;
	        }
	if(svc->current_state==STATE_CRITICAL && !svc->notify_on_critical){
#ifdef DEBUG4
		printf("\tWe shouldn't notify about CRITICAL states for this service!\n");
#endif
		return OK;
	        }
	if(svc->current_state==STATE_OK){
		if(!svc->notify_on_recovery){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about RECOVERY states for this service!\n");
#endif
			return OK;
		        }
		if((svc->last_hard_state==STATE_WARNING || svc->last_hard_state==STATE_UNKNOWN) && !svc->notify_on_warning){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about recoveries from warning or unknown states for this service!\n");
#endif
			return OK;
		        }
		if(svc->last_hard_state==STATE_CRITICAL && !svc->notify_on_critical){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about recoveries from critical states for this service!\n");
#endif
			return OK;
		        }
	        }


	/* see if the service can have notifications sent out at this time */
	if(!check_time_against_period((unsigned long)time(NULL),svc->notification_period)){
#ifdef DEBUG4
		printf("\tThis service shouldn't have notifications sent out at this time!\n");
#endif

		/* calculate the next acceptable notification time, once the next valid time range arrives... */
		get_next_valid_time(current_time,&timeperiod_start,svc->notification_period);
		svc->next_notification=timeperiod_start;

		return OK;
	        }

	/* next notification time should be 0 when the last check was okay, so the next if statement
	   is always true if a service just went down (i.e. the contacts *will* be notified) */
	if(svc->current_state==STATE_OK)
		svc->next_notification=(time_t)0;

	/* don't notify if we haven't waited long enough since the last time */
	if(current_time < svc->next_notification){
#ifdef DEBUG4
		printf("\tWe haven't waited long enough to re-notify contacts about this service!\n");
#endif
		return OK;
	        }

	/* calculate the next acceptable re-notification time */
	svc->next_notification=current_time+(svc->notification_interval*interval_length);

	/* grab the macro variables */
	grab_host_macros(temp_host);
	grab_service_macros(svc);

	/* create the contact notification list for this service */
	create_notification_list_from_service(svc);

	/* notify each contact (duplicates have been removed) */
	for(temp_notification=notification_list;temp_notification!=NULL;temp_notification=temp_notification->next){

		/* grab the macro variables for this contact */
		grab_contact_macros(temp_notification->contact);

		/* notify this contact */
		notify_contact_of_service(temp_notification->contact,svc);
	        }

	/* free memory allocated to the notification list */
	free_notification_list();

	/* update the last notification time for this service */
	svc->last_notification=current_time;

#ifdef DEBUG0
	printf("service_notification() end\n");
#endif

	return OK;
	}


/* notify a specific contact about a service problem or recovery */
int notify_contact_of_service(contact *cntct,service *svc){
	commandsmember *temp_commandsmember;
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char command_line[MAX_COMMANDLINE_LENGTH];
	char raw_summary[MAX_COMMANDLINE_LENGTH];
	char temp_command_line[MAX_COMMANDLINE_LENGTH];
	char temp_buffer[MAX_COMMANDLINE_LENGTH+MAX_INPUT_BUFFER];
	char service_state[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("notify_contact_of_service() start\n");
#endif
#ifdef DEBUG4
	printf("\tNotify user %s\n",cntct->name);
#endif


	/* see if we should notify about problems with this service */
	if((svc->current_state==STATE_WARNING || svc->current_state==STATE_UNKNOWN) && !cntct->notify_on_service_warning){
#ifdef DEBUG4
		printf("\tWe shouldn't notify this contact about WARNING or UNKNOWN states!\n");
#endif
		return OK;
	        }

	if(svc->current_state==STATE_CRITICAL && !cntct->notify_on_service_critical){
#ifdef DEBUG4
		printf("\tWe shouldn't notify this contact about CRITICAL states!\n");
#endif
		return OK;
	        }

	if(svc->current_state==STATE_OK){

		if(!cntct->notify_on_service_recovery){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about RECOVERY states!\n");
#endif
			return OK;
		        }

		if((svc->last_hard_state==STATE_WARNING || svc->last_hard_state==STATE_UNKNOWN) && !cntct->notify_on_service_warning){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about recoveries from warning or unknown states!\n");
#endif
			return OK;
		        }

		if(svc->last_hard_state==STATE_CRITICAL && !cntct->notify_on_service_critical){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about recoveries from critical states!\n");
#endif
			return OK;
		        }
	        }

	/* see if the contact can be notified at this time */
	if(!check_time_against_period((unsigned long)time(NULL),cntct->service_notification_period)){
#ifdef DEBUG4
		printf("\tThis contact shouldn't be notified at this time!\n");
#endif
		return OK;
	        }

	/* process all the notification commands this user has */
	for(temp_commandsmember=cntct->service_notification_commands;temp_commandsmember!=NULL;temp_commandsmember=temp_commandsmember->next){

		/* get the raw command line */
		strncpy(temp_command_line,temp_commandsmember->command,sizeof(temp_command_line));
		temp_command_line[sizeof(temp_command_line)-1]='\x0';
		get_raw_command_line(temp_command_line,&raw_command_line[0],(int)sizeof(raw_command_line));

		/* run the notification command */
		if(strcmp(raw_command_line,"")){

			/* create summary macro */
			strncpy(raw_summary,generic_summary,sizeof(raw_summary));
			raw_summary[sizeof(raw_summary)-1]='\x0';
			process_macros(raw_summary,&macro_summary[0],(int)sizeof(macro_summary));

#ifdef DEBUG4
			printf("\tRaw Command:       %s\n",raw_command_line);
#endif

			/* replace macros in the command line */
			process_macros(raw_command_line,&command_line[0],(int)sizeof(command_line));

#ifdef DEBUG4
			printf("\tProcessed Command: %s\n",command_line);
#endif

			if(svc->current_state==STATE_OK)
				strcpy(service_state,"RECOVERY");
			else if(svc->current_state==STATE_CRITICAL)
				strcpy(service_state,"CRITICAL");
			else if(svc->current_state==STATE_WARNING)
				strcpy(service_state,"WARNING");
			else
				strcpy(service_state,"UNKNOWN");

			/* log the notification to program log file */
			if(log_notifications){
				snprintf(temp_buffer,sizeof(temp_buffer),"SERVICE NOTIFICATION: %s;%s;%s;%s;%s;%s\n",cntct->name,svc->host_name,svc->description,service_state,temp_command_line,svc->plugin_output);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				write_to_log(temp_buffer);
			        }

			/* run the command */
			my_system(command_line);
		        }

	        }

#ifdef DEBUG0
	printf("notify_contact_of_service() end\n");
#endif

	return OK;
        }



/* given a service, create a list of contacts to be notified, removing duplicates */
int create_notification_list_from_service(service *svc){
	contactgroup *temp_contactgroup;
	contactgroupmember *temp_contactgroupmember;
	contact *temp_contact;
	char contact_groups[MAX_CONTACTGROUPS_LENGTH];
	char *temp_groupname;


	/* find all contact groups for this service */
	strcpy(contact_groups,svc->contact_groups);
	for(temp_groupname=my_strtok(contact_groups,",");temp_groupname!=NULL;temp_groupname=my_strtok(NULL,",")){

		/* find the contact group */
		temp_contactgroup=find_contactgroup(temp_groupname,NULL);

		/* if we couldn't find the contact group, skip it */
		if(temp_contactgroup==NULL)
			continue;

		/* all all contacts in this contact group to the notification list (ignoring duplicates) */
		temp_contactgroupmember=temp_contactgroup->members;
		while(temp_contactgroupmember!=NULL){

			/* add the contact to the notification list */
			temp_contact=NULL;
			temp_contact=find_contact(temp_contactgroupmember->contact_name,temp_contact);
			if(temp_contact!=NULL)
				add_notification(temp_contact);

			temp_contactgroupmember=temp_contactgroupmember->next;
		        }
	        }

	return OK;
        }




/******************************************************************/
/******************* HOST NOTIFICATION FUNCTIONS ******************/
/******************************************************************/

/* notify all contacts for a host that the entire host is down or up */
int host_notification(host *hst,int state){
	notification *temp_notification;
	time_t current_time;
	time_t timeperiod_start;
	int state_change;


#ifdef DEBUG0
	printf("host_notification() start\n");
#endif

	/* are we in active mode? */
	if(program_mode!=ACTIVE_MODE){
#ifdef DEBUG4
		printf("\tWe are in STANDBY mode, so host notifications will not be sent out!\n");
#endif
		return OK;
	        }

	/* are notifications temporarily disabled for this host? */
	if(hst->notifications_disabled==TRUE){
#ifdef DEBUG4
		printf("\tNotifications are temporarily disabled for this host, so we won't send one out!\n");
#endif
		return OK;
	        }

	/* get the current time */
	time(&current_time);

	/* has the host state changed? */
	if(hst->status!=state)
		state_change=TRUE;
	else
		state_change=FALSE;


	/* see if we should notify about problems with this host */
	if((state==HOST_UNREACHABLE) && !hst->notify_on_unreachable){
#ifdef DEBUG4
		printf("\tWe shouldn't notify about UNREACHABLE status for this host!\n");
#endif
		return OK;
	        }
	if(state==HOST_DOWN && !hst->notify_on_down){
#ifdef DEBUG4
		printf("\tWe shouldn't notify about DOWN states for this host!\n");
#endif
		return OK;
	        }
	if(state==HOST_UP){

		if(!hst->notify_on_recovery){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about RECOVERY states for this host!\n");
#endif
			return OK;
		       }

		if(hst->status==HOST_UNREACHABLE && !hst->notify_on_unreachable){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about recoveries from unreachable states for this host!\n");
#endif
			return OK;
		        }

		if(hst->status==HOST_DOWN && !hst->notify_on_down){
#ifdef DEBUG4
			printf("\tWe shouldn't notify about recoveries from down states for this host!\n");
#endif
			return OK;
		        }
	        }


	/* see if the host can have notifications sent out at this time */
	if(!check_time_against_period((unsigned long)time(NULL),hst->notification_period)){
#ifdef DEBUG4
		printf("\tThis host shouldn't have notifications sent out at this time!\n");
#endif

		/* calculate the next acceptable notification time, once the next valid time range arrives... */
		get_next_valid_time(current_time,&timeperiod_start,hst->notification_period);
		hst->next_host_notification=timeperiod_start;

		return OK;
	        }

	/* if we haven't notified the contacts about the host state change, do so */
	if(state_change==TRUE){

#ifdef DEBUG3
		printf("\tHOST STATE CHANGE!\n");
#endif
	        }

	/* else if its time to re-notify the contacts about the host... */
	else if(state!=HOST_UP && current_time >= hst->next_host_notification){

#ifdef DEBUG3
		printf("\tTime to re-notify the contacts about host being down!\n");
#endif
	        }

	/* else we shouldn't do the notification, so exit */
	else{

#ifdef DEBUG3
		printf("\tIts not yet time to re-notify the contacts about this host being down...\n");
#endif

		return OK;
	        }

	/* calculate the next acceptable re-notification time */
	hst->next_host_notification=current_time+(hst->notification_interval*interval_length);

	/* grab the macro variables */
	grab_host_macros(hst);

	/* make sure the host state macro is correct */
	if(state==HOST_DOWN)
		strcpy(macro_host_state,"DOWN");
	else if(state==HOST_UNREACHABLE)
		strcpy(macro_host_state,"UNREACHABLE");
	else
		strcpy(macro_host_state,"UP");

	/* create the contact notification list for this host */
	create_notification_list_from_host(hst);

	/* notify each contact (duplicates have been removed) */
	for(temp_notification=notification_list;temp_notification!=NULL;temp_notification=temp_notification->next){

		/* grab the macro variables for this contact */
		grab_contact_macros(temp_notification->contact);

		/* notify this contact */
		notify_contact_of_host(temp_notification->contact,hst,state);
	        }

	/* free memory allocated to the notification list */
	free_notification_list();

	/* update the last notification time for this host */
	hst->last_host_notification=current_time;

#ifdef DEBUG0
	printf("host_notification() end\n");
#endif

	return OK;
        }


/* notify a specific contact that an entire host is down or up */
int notify_contact_of_host(contact *cntct,host *hst,int state){
	commandsmember *temp_commandsmember;
	char temp_buffer[MAX_INPUT_BUFFER];
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char command_line[MAX_COMMANDLINE_LENGTH];
	char raw_summary[MAX_COMMANDLINE_LENGTH];
	char temp_command_line[MAX_COMMANDLINE_LENGTH];


#ifdef DEBUG0
	printf("notify_contact_of_host() start\n");
#endif
#ifdef DEBUG4
	printf("\tNotify user %s\n",cntct->name);
#endif

	/* see if we should notify about problems with this host */
	if((state==HOST_DOWN) && !cntct->notify_on_host_down){
#ifdef DEBUG4
		printf("\tWe shouldn't notify this contact about DOWN states!\n");
#endif
		return OK;
	        }

	if(state==HOST_UNREACHABLE && !cntct->notify_on_host_unreachable){
#ifdef DEBUG4
		printf("\tWe shouldn't notify this contact about UNREACHABLE states!\n");
#endif
		return OK;
	        }

	if(state==HOST_UP){

		if(!cntct->notify_on_host_recovery){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about RECOVERY states!\n");
#endif
			return OK;
		        }

		if(hst->status==HOST_DOWN && !cntct->notify_on_host_down){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about recoveries from down states!\n");
#endif
			return OK;
		        }

		if(hst->status==HOST_UNREACHABLE && !cntct->notify_on_host_unreachable){
#ifdef DEBUG4
			printf("\tWe shouldn't notify this contact about recoveries from unreachable states!\n");
#endif
			return OK;
		        }
	        }


	/* see if the contact can be notified at this time */
	if(!check_time_against_period((unsigned long)time(NULL),cntct->host_notification_period)){
#ifdef DEBUG4
		printf("\tThis contact shouldn't be notified at this time!\n");
#endif
		return OK;
	        }

	/* process all the notification commands this user has */
	for(temp_commandsmember=cntct->host_notification_commands;temp_commandsmember!=NULL;temp_commandsmember=temp_commandsmember->next){

		/* get the raw command line */
		strncpy(temp_command_line,temp_commandsmember->command,sizeof(temp_command_line));
		temp_command_line[sizeof(temp_command_line)-1]='\x0';
		get_raw_command_line(temp_command_line,&raw_command_line[0],(int)sizeof(raw_command_line));

		/* run the notification command */
		if(strcmp(raw_command_line,"")){

			/* create summary macro */
			strncpy(raw_summary,generic_summary,sizeof(raw_summary));
			raw_summary[sizeof(raw_summary)-1]='\x0';
			process_macros(raw_summary,&macro_summary[0],(int)sizeof(macro_summary));

#ifdef DEBUG4
			printf("\tRaw Command:       %s\n",raw_command_line);
#endif
			/* replace macros in the command line */
			process_macros(raw_command_line,&command_line[0],(int)sizeof(command_line));

#ifdef DEBUG4
			printf("\tProcessed Command: %s\n",command_line);
#endif

			/* log the notification to program log file */
			if(log_notifications){
				snprintf(temp_buffer,sizeof(temp_buffer),"HOST NOTIFICATION: %s;%s;%s;%s;Host is %s\n",cntct->name,hst->name,(state==HOST_UP)?"RECOVERY":macro_host_state,temp_command_line,macro_host_state);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				write_to_log(temp_buffer);
			        }

			/* run the command */
			my_system(command_line);
		        }
	        }


#ifdef DEBUG0
	printf("notify_contact_of_host() end\n");
#endif

	return OK;
        }


/* given a host, create a list of contacts to be notified, removing duplicates */
int create_notification_list_from_host(host *hst){
	hostgroup *temp_hostgroup;
	hostgroupmember *temp_hostgroupmember;
	contactgroup *temp_contactgroup;
	contactgroupmember *temp_contactgroupmember;
	contact *temp_contact;
	char contact_groups[MAX_CONTACTGROUPS_LENGTH];
	char *temp_groupname;

	/* check all the hostgroups for this host */
	temp_hostgroup=hostgroup_list;
	while(temp_hostgroup!=NULL){

		/* see if the host if a member of this hostgroup */
		temp_hostgroupmember=NULL;
		temp_hostgroupmember=find_hostgroupmember(hst->name,temp_hostgroup,temp_hostgroupmember);

		/* we found a hostgroup that includes this host */
		if(temp_hostgroupmember!=NULL){

			/* find all contact groups for this hostgroup */
			strcpy(contact_groups,temp_hostgroup->contact_groups);
			for(temp_groupname=my_strtok(contact_groups,",");temp_groupname!=NULL;temp_groupname=my_strtok(NULL,",")){

				/* find the contact group */
				temp_contactgroup=find_contactgroup(temp_groupname,NULL);

				/* if we couldn't find the contact group, skip it */
				if(temp_contactgroup==NULL)
					continue;

				/* all all contacts in this contact group to the notification list (ignoring duplicates) */
				temp_contactgroupmember=temp_contactgroup->members;
				while(temp_contactgroupmember!=NULL){

					/* add the contact to the notification list */
					temp_contact=NULL;
					temp_contact=find_contact(temp_contactgroupmember->contact_name,temp_contact);
					if(temp_contact!=NULL)
						add_notification(temp_contact);

					temp_contactgroupmember=temp_contactgroupmember->next;
				        }
			        }
	                }

		/* find the next hostgroup */
		temp_hostgroup=temp_hostgroup->next;
	        }

	return OK;
        }

